-------------------------------------------------------------------------------
-- MrCurrency
--
-- Data Broker source for Currency
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local MRCUtils = {};
LibStub("AceAddon-3.0"):GetAddon("MrCurrency").Utils = MRCUtils;

-- Useful for getting information on the currency itself
local Currencies = {
	["Interface\\Archeology\\Arch-Race-Dwarf"] = 384; -- Dwarf
	["Interface\\Archeology\\Arch-Race-Draenei"] = 398; -- Draenei
	["Interface\\Archeology\\Arch-Race-Fossil"] = 393; -- Fossil
	["Interface\\Archeology\\Arch-Race-NightElf"] = 394; -- Night Elf
	["Interface\\Archeology\\Arch-Race-Nerubian"] = 400; -- Nerubian
	["Interface\\Archeology\\Arch-Race-Orc"] = 397; -- Orc
	["Interface\\Archeology\\Arch-Race-Tolvir"] = 401; -- Tol'Vir
	["Interface\\Archeology\\Arch-Race-Troll"] = 385; -- Troll
	["Interface\\Archeology\\Arch-Race-Vrykul"] = 399; -- Vrykul
};

function MRCUtils:GetCurrencyIcon(raceIcon)
	local tokenCurrency = Currencies[raceIcon];
	if( tokenCurrency ~= nil ) then
		local _, _, tokenIcon = GetCurrencyInfo(tokenCurrency);
		
		if( tokenIcon ~= nil ) then
			return "Interface\\Icons\\" .. tokenIcon;
		end
	end
	
	return raceIcon;
end

function MRCUtils:ReadCurrency()
	local result = {};
	
	-- Expand all Items
	local expanding = true;
	local expandedItems = {};
	while expanding do
		expanding = false;
		for index=1,GetCurrencyListSize() do
			name, isHeader, isExpanded, isUnused, isWatched, count, icon, version = GetCurrencyListInfo(index);
		
			if( isHeader and not isExpanded ) then
				ExpandCurrencyList(index, 1);
				tinsert(expandedItems, 1, index);
				expanding = true;
			end
		end
	end
	
	-- Process
	for index=1,GetCurrencyListSize() do
		name, isHeader, isExpanded, isUnused, isWatched, count, icon, version = GetCurrencyListInfo(index);

		if( name and not isHeader ) then
			local item = {
				name = name,
				isUnused = isUnused,
				isWatched = isWatched,
				count = count,
				icon = icon,
			};
			tinsert(result, item);
		end	
	end
	
	-- Recollapse
	for _,item in ipairs(expandedItems) do
		ExpandCurrencyList(item, 0);
	end
	
	return result;
end

function MRCUtils:AppendMoneyString(text, money, size, yOffset, padNumbers)
	local gold, silver, copper = MRCUtils:GetMoneySplit(money);
	local text = text or "";
	local shortMoneyFormat = "%d";
	local moneyFormat = shortMoneyFormat;
	
	if(padNumbers) then
		moneyFormat = "%02d";
	end

	text = MRCUtils:AppendCurrency(text, string.format(shortMoneyFormat, gold), "Interface\\MoneyFrame\\UI-GoldIcon", 1, size, yOffset);
	text = MRCUtils:AppendCurrency(text, string.format(moneyFormat, silver), "Interface\\MoneyFrame\\UI-SilverIcon", 1, size, yOffset);
	text = MRCUtils:AppendCurrency(text, string.format(moneyFormat, copper), "Interface\\MoneyFrame\\UI-CopperIcon", 1, size, yOffset);
	
	return text;	
end

function MRCUtils:AppendCurrency(text,amount,icon,displayZero,size,yOffset,padding)
	if(amount == nil) then amount = 0 end;
	if(displayZero == nil) then displayZero = false end;
	
	size = size or 18;
	yOffset = yOffset or 0;
	
	if( tonumber(amount) > 0 or displayZero ) then
		if( text ~= "" ) then text = text .. " " end;
		text = text .. amount .. " |T" .. icon .. ":" .. size .. ":" .. size .. ":0:" .. yOffset .. ":64:64:4:60:4:60|t";
	end
	
	return text;
end

function MRCUtils:GetMoneySplit(money)
	local gold, silver, copper;
	
	if(money == nil) then
		money = GetMoney();
	end
	
	copper = floor(money % 100);
	money = floor(money / 100);
	silver = money % 100;
	money = floor(money / 100);
	gold = money;
	
	return gold, silver, copper;
end

function MRCUtils:TablePairPack(...)
	local t,k,v;
	t = {};
	for i = 1, select("#", ...), 2 do
		k = select(i, ...);
		v = select(i+1, ...) or 0;
		tinsert(t, k);
		tinsert(t, v);
	end
	return t;
end

function MRCUtils:TablePack(...)
    local t,v;
    t = {};
    for i = 1, select("#", ...) do
        v = select(i, ...);
		if(v ~= nil) then
        	tinsert(t, v);
		end
    end
    return t;
end

function MRCUtils:TableSize(table)
	local count = 0;

	for _, _ in pairs(table) do
		count = count + 1;
	end
	
	return count;
end

function MRCUtils:TableJoin(t1, t2)
    for k,v in pairs(t2) do
		t1[k] = v;
	end 
	return t1;  
end

local function __genOrderedIndex( t )
    local orderedIndex = {}
    for key in pairs(t) do
        table.insert( orderedIndex, key )
    end
    table.sort( orderedIndex )
    return orderedIndex
end

local function orderedNext(t, state)
    -- Equivalent of the next function, but returns the keys in the alphabetic
    -- order. We use a temporary ordered key table that is stored in the
    -- table being iterated.

    --print("orderedNext: state = "..tostring(state) )
    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndex( t )
        key = t.__orderedIndex[1]
        return key, t[key]
    end
    -- fetch the next value
    key = nil
    for i = 1,table.getn(t.__orderedIndex) do
        if t.__orderedIndex[i] == state then
            key = t.__orderedIndex[i+1]
        end
    end

    if key then
        return key, t[key]
    end

    -- no more value to return, cleanup
    t.__orderedIndex = nil
    return
end

function MRCUtils:OrderedPairs(t)
    -- Equivalent of the pairs() function on tables. Allows to iterate
    -- in order
    return orderedNext, t, nil
end